import ast
import importlib
import inspect
import pprint

# TODO: Find full paths to callbacks
# TODO: Parse class instantiation for callbacks
# TODO: Handle pattern expressions instead of just strings
# TODO: Split into mutiple functions

def parse_pattern_regex(node):
    if isinstance(node, ast.Str):
        return node.s
    print ast.dump(node)
    print compile(node, '<string>', 'eval')
    # return ast.literal_eval(node)
    raise Exception('CRAP')

def parse_patterns(root_urlconf, url_prefix=''):
    """
    Import and parse urlpatterns from the given path.
    """
    module = importlib.import_module(root_urlconf)
    filename = inspect.getsourcefile(module)
    with open(filename) as fh:
        source = fh.read()

    context = {}
    tree = ast.parse(source)
    for node in ast.iter_child_nodes(tree):
        if isinstance(node, ast.Assign) and node.targets[0].id != 'urlpatterns':
            context[node.targets[0].id] = node.value
        if isinstance(node, ast.Assign) and node.targets[0].id == 'urlpatterns':
            prefix = node.value.args[0].s
            for arg in node.value.args[1:]:
                if isinstance(arg, ast.Call):
                    components = arg.args
                elif isinstance(arg, ast.Tuple):
                    components = arg.elts
                else:
                    raise Exception('Invalid argument to urlpatterns was found.')

                if len(components) == 2:
                    regex_node, callback_node = components
                    default_args_node = None
                elif len(components) == 3:
                    regex_node, callback_node, default_args_node = components

                if isinstance(callback_node, ast.Attribute):
                    callback = '.'.join((callback_node.value.id, callback_node.attr))
                    pprint.pprint({
                        'url_prefix': url_prefix,
                        'prefix': prefix,
                        'line': arg.lineno,
                        'filename': filename,
                        'regex': parse_pattern_regex(regex_node),
                        'callback': callback,
                    })
                elif isinstance(callback_node, ast.Call):
                    if isinstance(callback_node.func, ast.Name) and callback_node.func.id == 'include':
                        url_prefix = parse_pattern_regex(regex_node)
                        included_urlconf_path = callback_node.args[0].s
                        parse_patterns(included_urlconf_path, url_prefix)
                        # print ast.dump(callback_node)
                        # print ast.dump(callback_node)
        print context

if __name__ == '__main__':
    parse_patterns('alto.tests.urls')
    # parse_patterns('everyblock.web.urls')



import os
from django.conf import settings
from django.template import loader, TemplateDoesNotExist
from django.template.loader_tags import ExtendsNode


def find_template_source(name):
    """
    Load the source code and origin of the first template that matches the
    given name.
    """
    for loader_path in settings.TEMPLATE_LOADERS:
        template_loader = loader.find_template_loader(loader_path)
        try:
            source, origin = template_loader.load_template_source(name)
        except TemplateDoesNotExist:
            continue
        break
    return source, origin

def find_parents(name, parents=None):
    """
    Recursively find all of this template's parents and return them as a list.
    """
    template = loader.get_template(name)
    source, origin = find_template_source(name)
    if parents is None:
        parents = []
    else:
        parents.append({'name': name, 'file': origin})
    for node in template:
        if isinstance(node, ExtendsNode):
            parent_name = node.parent_name.token.strip('"')
            return find_parents(parent_name, parents)
    return parents

def find_template(name):
    source, origin = find_template_source(name)
    parents = list(reversed(find_parents(name)))
    return {'name': name, 'file': origin, 'source': source, 'parents': parents}

def find_templates():
    # Find all possible template directories.
    loaders = []
    for loader_path in settings.TEMPLATE_LOADERS:
        loaders.append(loader.find_template_loader(loader_path))

    paths = []
    for template_loader in loaders:
        temp_name = '__TEMP__.HTML'
        for path in template_loader.get_template_sources(temp_name):
            paths.append(path.strip(temp_name))

    # Find all possible templates in those template directories.
    for root_path in paths:
        for dirpath, dirnames, filenames in os.walk(root_path, followlinks=True):
            relative_path = os.path.relpath(dirpath, root_path)
            for filename in filenames:
                name = os.path.normpath(os.path.join(relative_path, filename))
                path = os.path.join(dirpath, filename)
                yield {'name': name, 'path': path}

if __name__ == '__main__':
    import json
    templates = list(find_templates())
    print json.dumps(templates, sort_keys=True, indent=2)



import json
import os
from django import http
from django.conf import settings
from django.template import RequestContext
from django.template.loaders.filesystem import Loader
from alto import urlviz
from alto.templates import find_template, find_templates

ALTO_TEMPLATE_DIRS = [os.path.join(os.path.dirname(__file__), 'templates')]

def index(request, mode=None):
    # Use the filesystem loader directly here so we don't need to require
    # people to add the app_directories template loader if they don't want to.
    loader = Loader()
    template, name = loader('alto/index.html', template_dirs=ALTO_TEMPLATE_DIRS)
    context = RequestContext(request, {
        'url_scheme': getattr(settings, 'ALTO_URL_SCHEME', 'mvim'),
        'query': request.GET.get('q', ''),
        'mode': mode or '',
        'STATIC_URL': '/_alto/media/',
    })
    return http.HttpResponse(template.render(context))

def url_patterns(request):
    patterns = urlviz.inspect_urlpatterns()
    return http.HttpResponse(json.dumps(patterns, sort_keys=True, indent=2))

def view_detail(request, module_path, view_name):
    view = urlviz.load_view(module_path, view_name)
    view, decorators = urlviz.extract_view(view)
    data = urlviz.inspect_view(view)
    return http.HttpResponse(json.dumps(data, sort_keys=True, indent=2))

def templates(request):
    data = list(find_templates())
    return http.HttpResponse(json.dumps(data, sort_keys=True, indent=2))

def template_detail(request, name):
    data = find_template(name)
    return http.HttpResponse(json.dumps(data, sort_keys=True, indent=2))



import os
from django.conf.urls import *
from alto import views

urlpatterns = patterns('',
    url(r'views/([\w\d\a\._]+)/([\w\d_]+)/$', views.view_detail),
    url(r'templates/(.+)/$', views.template_detail),
    url(r'urlpatterns/$', views.url_patterns),
    url(r'template-paths/$', views.templates),
    url(r'(?P<mode>\w+)/$', views.index),
    url(r'^media/(?P<path>.*)$', 'django.views.static.serve', {
        'document_root': os.path.join(os.path.dirname(__file__), 'static'),
    }),
    url(r'$', views.index),
)



import os
from setuptools import setup

README = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()

setup(
    name = 'alto',
    version = '0.4',
    description = 'A high-level code browser for Django projects.',
    long_description = README,
    author = 'Joseph Kocherhans',
    author_email = 'joseph@jkocherhans.com',
    license='BSD',
    url='http://github.com/jkocherhans/alto',
    packages = ['alto'],
    include_package_data = True,
    classifiers = [
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
    ],
    install_requires=[
        'Django>=1.4'
    ]
)
